@@ -16,6 +16,12 @@ import records #record data
 #own functions
 from functions_general import *
 
+#own c cfunctions (for speed improvements)
+try:
+    from lib import mylib
+except:
+    pass #already handled in main.py
+
 
 ###########################################################################
 ## Read the sensor data x times and smoothen it
@@ -90,8 +96,10 @@ def GetFirstEntry():
 ###########################################################################
 ##  Process the data, generate movement indication
 ###########################################################################
-def ProcessData(ProcessAll=False):  #DERIVATION (ABLEITUNG)
+def ProcessData(ProcessAll=False, ShowFiltered=False):  #DERIVATION (ABLEITUNG)
+#    global ViewWindow
     if(data.debug==1): print "Process Data (Process whole array:", ProcessAll,  ")"
+    data.Progress="Process data..."
     lastmovebeforegraph=0 
     if(ProcessAll==True): start=0 #process whole array
     else: start=records.Index-data.Derivation-1 #process only last (changed) records
@@ -113,8 +121,54 @@ def ProcessData(ProcessAll=False):  #DERIVATION (ABLEITUNG)
         if(records.arrData[3][i]>data.Trigger): #over trigger
             lastindexovertrigger=i
 
-#    for i in range(0,  records.Index-data.Derivation):     
-#        print i,  records.arrData[0][i], records.arrData[3][i],  records.arrData[4][i]
+
+
+
+
+def FilterData():
+
+###        mylib.sleep_filter()
+##        if(data.debug==1): print "Filter array:"
+##        if(data.debug==1): print data.arrFilter
+##        l=len(data.arrFilter)
+##        if(data.debug==1): print "Lenght:",  l
+##        
+        Xscale=data.arrXscale[data.Xscale]
+##        
+##        for i in range(0,  records.Index-data.Derivation,  Xscale): #loop arrData
+##            value=records.arrData[3][i]
+##            if((i%500)==0): 
+##                Progress="Filter data ("+ str(i*100/(records.Index-data.Derivation)) +"%)..."
+#        self.ui.lblStatus.setText("Apply filter")     
+#        app.processEvents() #force ui update
+##            for p in range(0,  l-1): #loop arrFilter
+##                records.arrData[6][i+p-data.FilterShift] = records.arrData[6][i+p-data.FilterShift] + value*data.arrFilter[p]
+##
+##        for i in range(0,  records.Index,  Xscale): #scalling
+##            records.arrData[6][i]  = records.arrData[6][i] / data.FilterScale*Xscale
+##            if(records.arrData[6][i] >data.Ysize):
+##                records.arrData[6][i]  = int(random.uniform(data.Ysize-9,data.Ysize-15))
+        
+#        a=mylib.sleep_filter()
+#        print a
+
+#        print "##### mylib is compiled:", mylib.IsCompiled()
+
+        arrSend=[0]*(records.Index+len(data.arrFilter))
+        for i in range(0,  records.Index-data.Derivation): #copy array
+            arrSend[i]=records.arrData[3][i]
+        
+#        for i in range(0,  50): print arrSend[i],  records.arrData[3][i]
+        
+        arrRet=mylib.sleep_filter(arrSend, records.Index-data.Derivation, data.arrFilter, data.FilterShift)
+#        print arrRet
+        
+        for i in range(0,  records.Index-data.Derivation): #copy array and scale
+            records.arrData[6][i]  = arrRet[i] / data.FilterScale
+            if(records.arrData[6][i] > data.Ysize-7):
+                records.arrData[6][i]  = data.Ysize-7 #-int(random.uniform(1, 2))
+            elif(records.arrData[6][i] < 13): records.arrData[6][i] =13
+
 
 
 
@@ -125,10 +179,10 @@ def ProcessData(ProcessAll=False):  #DERIVATION (ABLEITUNG)
 ## Show (refreshs) the graph.
 ## the first parameter indcates on which window it will be drawn
 ###########################################################################
-def DrawGraph(self,  QtGui):
+def DrawGraph(self,  QtGui, ShowFiltered=False):
     try:
         if(data.debug==1): print "Draw Graph into bitmap"
-
+        
         Xscale=data.arrXscale[data.Xscale]
         decade=360/Xscale
         
@@ -153,7 +207,7 @@ def DrawGraph(self,  QtGui):
     #    print "data.QuickMode:",  data.QuickMode
             
     #    data.ChangesInGraph=False
-        paint = QtGui.QPainter()
+        paint = QtGui.QPainter()     
         paint.begin(self)
 
         paint.setFont(QtGui.QFont('Decorative', 12))
@@ -273,6 +327,17 @@ def DrawGraph(self,  QtGui):
 
 
         #DRAW time bars
+        State1Color=QtGui.QColor(0, 200, 0)
+        State1ColorDark=QtGui.QColor(0, 50, 0)
+        State2Color=QtGui.QColor(0, 100, 0)
+        State2ColorDark=QtGui.QColor(0, 20, 0)
+        State3Color=QtGui.QColor(174, 57, 16)
+        State3ColorDark=QtGui.QColor(88, 29, 8)
+        State4Color=QtGui.QColor(134, 0, 0)
+        State4ColorDark=QtGui.QColor(76, 0, 0)
+        
+        gradient = QtGui.QLinearGradient(0, data.Ysize-11, 0, data.Ysize-1)
+        
         start=GraphStart_index % 6
         lastmoveovertrigger=start
     #    for p in range(start,  records.Index-data.Derivation):
@@ -289,13 +354,34 @@ def DrawGraph(self,  QtGui):
                 if(x0<x1): 
                     t=(x1-x0)*Xscale/6        
     #                print x0,  x1
-                    if(t>30): color=data.State1Color #green for 30 minutes and longer
-                    elif(t>15): color=data.State2Color #dark green for 15 .. 29 minutes
-                    elif(t>7): color=data.State3Color #orange for 7 .. 14 minutes
-                    else: color=data.State4Color #dark red for under 7 minutes
-                    paint.setBrush(QtGui.QColor(color))
-                    paint.setPen(QtGui.QColor(color))
-                    paint.drawRect(x0+1, data.Ysize-11, x1-x0-1, 10)                     
+#                    if(t>30): color=State1Color #green for 30 minutes and longer
+#                    elif(t>15): color=State2Color #dark green for 15 .. 29 minutes
+#                    elif(t>7): color=State3Color #orange for 7 .. 14 minutes
+#                    else: color=State4Color #dark red for under 7 minutes
+                    if(t>30): #green for 30 minutes and longer
+                        gradient.setColorAt(0, State1Color)
+                        gradient.setColorAt(0.5, State1ColorDark)
+                        gradient.setColorAt(1, State1Color)
+                        paint.setPen(QtGui.QColor(State1Color))
+                    elif(t>15): #dark green for 15 .. 29 minutes
+                        gradient.setColorAt(0, State2Color)
+                        gradient.setColorAt(0.5, State2ColorDark)
+                        gradient.setColorAt(1, State2Color)
+                        paint.setPen(QtGui.QColor(State2Color))
+                    elif(t>7): #orange for 7 .. 14 minutes
+                        gradient.setColorAt(0, State3Color)
+                        gradient.setColorAt(0.5, State3ColorDark)
+                        gradient.setColorAt(1, State3Color)
+                        paint.setPen(QtGui.QColor(State3Color))
+                    else:  #dark red for under 7 minutes
+                        gradient.setColorAt(0, State4Color)
+                        gradient.setColorAt(0.5, State4ColorDark)
+                        gradient.setColorAt(1, State4Color)
+                        paint.setPen(QtGui.QColor(State4Color))
+                    paint.setBrush(QtGui.QBrush(gradient))
+#                    paint.setBrush(QtGui.QColor(color))
+#                    paint.setPen(QtGui.QColor(data.GraphBackground))
+                    paint.drawRect(x0+1, data.Ysize-11, x1-x0-1, 9)                     
                     paint.setPen(QtGui.QColor(data.MovementBarColor))
 
 
@@ -329,12 +415,42 @@ def DrawGraph(self,  QtGui):
 
 
 
+
+        #DRAW Filtered graph  
+        if(ShowFiltered==True):  #only show in view window, not during recording
+            pen=QtGui.QPen()    
             
+            #filling
+            pen.setColor(QtGui.QColor(19, 54, 255, 150))
+            pen.setWidth(1)
+            paint.setPen(pen)            
+            for p in range(0,  records.Index-data.Derivation-Xscale,  Xscale):
+                i=p-GraphStart_index
+                x=i/Xscale+1
+                paint.drawLine(x, data.Ysize-16, x, data.Ysize-records.arrData[6][p+Xscale]-5)  
+
+            #outline
+            pen.setColor(QtGui.QColor(19, 54, 255))
+            pen.setWidth(3)
+            paint.setPen(pen)            
+            for p in range(0,  records.Index-data.Derivation-Xscale,  Xscale):
+                i=p-GraphStart_index
+                x=i/Xscale+1
+                paint.drawLine(x, data.Ysize-records.arrData[6][p]-5, x+1, data.Ysize-records.arrData[6][p+Xscale]-5)  
+
+
+
+
+
+
+
+
+        
         
        #ALARM VISUALISATION
         if(records.AlarmIsSet==True): #alarm time window is set, show it            
-            paint.setBrush(QtGui.QColor(data.AlarmTimeWindowColor))
-            paint.setPen(QtGui.QColor(data.AlarmTimeWindowColor))  
+            paint.setBrush(QtGui.QColor(255, 128, 0, 200))
+            paint.setPen(QtGui.QColor(255, 128, 0, 200))  
             x0=Tuple2Index(records.AlarmTimeWindowStart_tuple)-data.Derivation*Xscale
             x1=Tuple2Index(records.AlarmTimeWindowEnd_tuple)-data.Derivation*Xscale
     #        x0=Tuple2Index(records.AlarmTimeWindowStart_tuple)-data.Derivation
@@ -361,8 +477,8 @@ def DrawGraph(self,  QtGui):
 #                    print "AlarmTimeWindowEnd:",  records.AlarmTimeWindowEnd_seconds,  "AlarmTime:",  records.AlarmTime_seconds,  "records.EndTime_seconds:",records.EndTime_seconds  
     #                EndTime_seconds=Tuple2Seconds(records.StartTime_tuple)+records.Index*records.Sampling_interval*records.Value_smoothing/1000
                     if(records.EndTime_seconds>=records.AlarmTime_seconds-60): #alarm got set off during recording (-60 is workaround)
-                        paint.setBrush(QtGui.QColor(data.AlarmColor))
-                        paint.setPen(QtGui.QColor(data.AlarmColor))  
+#                        paint.setBrush(QtGui.QColor(data.AlarmColor))
+                        paint.setPen(QtGui.QColor(178, 0, 0))  
                         if(records.AlarmTimeWindowEnd_seconds<=records.AlarmTime_seconds):
 #                            print "Alarm due end of alarm time window"
                             a0=Tuple2Index(records.AlarmTime_tuple)-data.Derivation*Xscale
@@ -370,7 +486,23 @@ def DrawGraph(self,  QtGui):
 #                            print "alarm due movement"
                             a0=Tuple2Index(records.AlarmTime_tuple)-data.Derivation
                         a00=(a0-GraphStart_index)/Xscale
+                        
+
+
+
+#                        gradient = QtGui.QRadialGradient(200,  100,  5)
+                        gradient = QtGui.QRadialGradient(a00,  20,  3)
+                        gradient.setColorAt(0, QtGui.QColor(255, 152, 152))
+                        gradient.setColorAt(1, QtGui.QColor(255, 0, 0))
+                        paint.setBrush(QtGui.QBrush(gradient))
+                        
+                        
+                        
+                        
+                        
+#                        paint.drawEllipse(100, 0, 200, 200) 
                         paint.drawEllipse(a00-5, 15, 10, 10)     
+                        
             #            print "AlarmTimeWindow:",  x1,  x10,  "Alarm:",  a0,  a00
 #                    else:
 #                        print "Alarm set, but never occured"
@@ -386,7 +518,7 @@ def DrawGraph(self,  QtGui):
                 paint.drawLine(1, y-15, data.GraphBitMapLen-2, y-15)  #Trigger line
                 paint.drawText(data.GraphBitMapLen-25, y-17, str(records.Trigger)) #Trigger level
     #            paint.drawText(5, 15, str(data.Yscale)) #Y scale
-            paint.setPen(QtGui.QColor("white"))
+            paint.setPen(QtGui.QColor(255,  255,  255, 200))
             y=(data.Ysize-(data.Trigger*data.Yscale/2))-2
             paint.drawLine(1, y-15, data.GraphBitMapLen-2, y-15)  #Trigger line
             paint.drawText(data.GraphBitMapLen-25, y-17, str(data.Trigger)) #Trigger level
@@ -396,10 +528,22 @@ def DrawGraph(self,  QtGui):
            
         #Draw border around graph
         paint.setPen(QtGui.QColor("white"))
-        paint.drawLine(0, 0, 0, data.Ysize)  #left side    
-        paint.drawLine(data.GraphBitMapLen-1, 0, data.GraphBitMapLen-1, data.Ysize)  #right side
-        paint.drawLine(0, 0, data.GraphBitMapLen-1, 0)  #top
-        paint.drawLine(0, data.Ysize, data.GraphBitMapLen-1, data.Ysize)  #bottom
+#        paint.drawLine(0, 0, 0, data.Ysize)  #left side    
+#        paint.drawLine(data.GraphBitMapLen-1, 0, data.GraphBitMapLen-1, data.Ysize)  #right side
+#        paint.drawLine(0, 0, data.GraphBitMapLen-1, 0)  #top
+#        paint.drawLine(0, data.Ysize, data.GraphBitMapLen-1, data.Ysize)  #bottom
+
+    #    #show Graph border 
+        paint.setPen(QtGui.QColor("white"))
+        paint.setBrush(QtGui.QColor(10, 10, 10, 200))   
+        
+        gradient = QtGui.QLinearGradient(0, 0, 0, data.Ysize)
+        gradient.setColorAt(0, QtGui.QColor(255, 255, 255, 50)) #top
+        gradient.setColorAt(0.5, QtGui.QColor(255, 255, 255, 0)) #bottom
+        paint.setBrush(QtGui.QBrush(gradient)) 
+        
+        
+        paint.drawRect(0,0,data.GraphBitMapLen-1,data.Ysize)   
 
 
            
@@ -414,6 +558,7 @@ def DrawGraph(self,  QtGui):
 
     except:
         print "Error while drawing the graph"
+        print sys.exc_info() 
 
     paint.end()
 
@@ -426,7 +571,7 @@ def DrawGraph(self,  QtGui):
 ###########################################################################
 ## 
 ###########################################################################  
-def DrawStatistic(self,  QtGui):
+def DrawStatistic(self,  QtGui, QtCore):
 #    try:
 #        if(daDta.debug==1): print "Draw Graph into bitmap"
 
@@ -435,24 +580,77 @@ def DrawStatistic(self,  QtGui):
     
     #clear whole pitmap (remove old data)
     paint.setPen(QtGui.QColor(data.GraphBackground))
-    paint.setBrush(QtGui.QColor(data.GraphBackground))    
+#    paint.setBrush(QtGui.QColor("red"))   
+#    paint.drawRect(0, 0, data.StatisticsBitMapLen/2,  data.StatisticsBitMapHeight/2)  
+    
+    paint.setBrush(QtGui.QColor(data.GraphBackground))   
+#    paint.setBrush(QtGui.QColor(255, 255, 255, 50))   
+#    paint.setBrush(QtCore.Qt.NoBrush)    
     paint.drawRect(0, 0, data.StatisticsBitMapLen,  data.StatisticsBitMapHeight)   
     
+#    paint.
+
+#    linearGrad = QtGui.QLinearGradient(10, 10, 20, 20)
+#    linearGrad.setColorAt(0, QtGui.QColor("red"))
+#    linearGrad.setColorAt(1, QtGui.QColor("green"))
+
+
+
+
+    
+
+
+
+
+
+
+
+
+#    paint.setBrush(QtGui.QGradient())
+
+    paint.setPen(QtGui.QColor(data.GraphBackground))
+    State1Color=QtGui.QColor(0, 200, 0)
+    State1ColorDark=QtGui.QColor(0, 50, 0)
+    State2Color=QtGui.QColor(0, 100, 0)
+    State2ColorDark=QtGui.QColor(0, 20, 0)
+    State3Color=QtGui.QColor(174, 57, 16)
+    State3ColorDark=QtGui.QColor(88, 29, 8)
+    State4Color=QtGui.QColor(134, 0, 0)
+    State4ColorDark=QtGui.QColor(76, 0, 0)
+
+
     if(records.arrStatData[3][3]>0):
-        paint.setPen(QtGui.QColor(data.State4Color))
-        paint.setBrush(QtGui.QColor(data.State4Color))    
+        a=QtGui.QColor(134, 0, 0)
+        gradient = QtGui.QLinearGradient(0, 5, 0, 15)
+        gradient.setColorAt(0, State4Color)
+        gradient.setColorAt(0.5, State4ColorDark)
+        gradient.setColorAt(1, State4Color)
+        paint.setBrush(QtGui.QBrush(gradient))
+#        paint.setPen(QtGui.QColor(State4Color))
         paint.drawRect(0, 5, records.arrStatData[3][3], 10)   
     if(records.arrStatData[2][3]>0):
-        paint.setPen(QtGui.QColor(data.State3Color))
-        paint.setBrush(QtGui.QColor(data.State3Color))    
+        gradient = QtGui.QLinearGradient(0, 20, 0, 30)
+        gradient.setColorAt(0, State3Color)
+        gradient.setColorAt(0.5, State3ColorDark)
+        gradient.setColorAt(1, State3Color)
+        paint.setBrush(QtGui.QBrush(gradient)) 
+#        paint.setPen(QtGui.QColor(State3Color))
         paint.drawRect(0, 20, records.arrStatData[2][3], 10)   
-    if(records.arrStatData[1][3]>0):
-        paint.setPen(QtGui.QColor(data.State2Color))
-        paint.setBrush(QtGui.QColor(data.State2Color))    
+    if(records.arrStatData[1][3]>0):  
+        gradient = QtGui.QLinearGradient(0, 35, 0, 45)
+        gradient.setColorAt(0, State2Color)
+        gradient.setColorAt(0.5, State2ColorDark)
+        gradient.setColorAt(1, State2Color)
+        paint.setBrush(QtGui.QBrush(gradient)) 
+#        paint.setPen(QtGui.QColor(State2Color))
         paint.drawRect(0, 35, records.arrStatData[1][3], 10)   
     if(records.arrStatData[0][3]>0):
-        paint.setPen(QtGui.QColor(data.State1Color))
-        paint.setBrush(QtGui.QColor(data.State1Color))    
+        gradient = QtGui.QLinearGradient(0, 50, 0, 60)
+        gradient.setColorAt(0, State1Color)
+        gradient.setColorAt(0.5, State1ColorDark)
+        gradient.setColorAt(1, State1Color)
+        paint.setBrush(QtGui.QBrush(gradient)) 
+#        paint.setPen(QtGui.QColor(State1Color))
         paint.drawRect(0, 50, records.arrStatData[0][3], 10)   
     
     paint.end()
@@ -592,6 +790,7 @@ def InitRecord():
     for i in range(0, records.MaxArrayDataLen-1): #clear data of old records
         for j in range(0,  4): 
             records.arrData[j][i]=0
+        records.arrData[6][i]=0
         records.arrData[5][i]=""
     
     print "Reset/cleared all record data"
